<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML//EN">
<html>
  <head>
    <title>Directory Listing of @LISTED_DIRECTORY@</title>
  </head>
    @INCLUDE bodytag.html@  
    <h1>@LISTED_DIRECTORY@</h1>
    <hr>
    <table border=0 cellspacing=0 cellpadding=3>
      <tr><td>Filename</td><td>Size</td></tr>
      @FOREACH i FILES@
      @ODD i@
      <tr bgcolor=#333333><td><a href="/playlist.pls?mount=/file/@i.urlname@&file=dummy.pls">@i.name@</a></td><td>@i.size@</td></tr>
      @EVEN i@
      <tr bgcolor=#000000><td><a href="/playlist.pls?mount=/file/@i.urlname@&file=dummy.pls">@i.name@</a></td><td>@i.size@</td></tr>
      @ENDFOR i@
</table>

    <hr>
  </body>
</html>
