<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML//EN">
<html>
  <head>
    <title>Welcome to the Icecast WWW Admin Interface</title>
  </head>
    @INCLUDE bodytag.html@
  <h1> Delete a mountpoint alias </h1>
  <hr>	
  <form action="/admin" method="get" align="center">
                                <table align=center>
                                        <tr><td align=left>Alias Name</td>
                                                  <td align=right><input size="30" type="text" name="argument1"></td>
                                        </tr>
					<input type=hidden name="mode" value="alias">
					<input type=hidden name="argument" value="del">
                                        </tr>
					<tr><td align=left><input type="submit" value="Delete"></td></tr>
				</table>
  </form>
</body>
</html>

